@@ -1,8 +1,9 @@
+import logging
 from collections import defaultdict
 from dataclasses import dataclass
 
 from django.contrib import admin
-from django.contrib.admin.options import BaseModelAdmin
+from django.contrib.admin.options import BaseModelAdmin, InlineModelAdmin
 from django.contrib.admin.utils import flatten_fieldsets
 from django.db import models
 from django.db.models.fields.reverse_related import ForeignObjectRel
@@ -22,24 +23,21 @@ from .query import (
 
 _OPEN_IN_ADMIN = "admin"
 
-_FIELD_MAP = [
-    ((models.BooleanField, models.NullBooleanField), BooleanFieldType),
-    (
-        (
-            models.CharField,
-            models.TextField,
-            models.GenericIPAddressField,
-            models.UUIDField,
-        ),
-        StringFieldType,
-    ),
-    ((models.DateTimeField, models.DateField), TimeFieldType),
-    (
-        (models.DecimalField, models.FloatField, models.IntegerField, models.AutoField),
-        NumberFieldType,
-    ),
-    ((models.FileField,), None),
-]
+_FIELD_MAP = {
+    models.BooleanField: BooleanFieldType,
+    models.NullBooleanField: BooleanFieldType,
+    models.CharField: StringFieldType,
+    models.TextField: StringFieldType,
+    models.GenericIPAddressField: StringFieldType,
+    models.UUIDField: StringFieldType,
+    models.DateTimeField: TimeFieldType,
+    models.DateField: TimeFieldType,
+    models.DecimalField: NumberFieldType,
+    models.FloatField: NumberFieldType,
+    models.IntegerField: NumberFieldType,
+    models.AutoField: NumberFieldType,
+    models.FileField: None,
+}
 
 
 def get_model_name(model, sep="."):
@@ -57,6 +55,7 @@ class OrmModel:
 class OrmField:
     type_: FieldType
     concrete: bool
+    model_name: str
 
 
 @dataclass
@@ -67,10 +66,10 @@ class OrmFkField:
 def _get_all_admin_fields(request):
     request.data_browser = True
 
-    def from_fieldsets(admin, model):
-        obj = model()  # we want the admin change field sets, not the add ones
+    def from_fieldsets(admin, all_):
+        obj = admin.model()  # we want the admin change field sets, not the add ones
         for f in flatten_fieldsets(admin.get_fieldsets(request, obj)):
-            if hasattr(model, f):
+            if not isinstance(admin, InlineModelAdmin) or hasattr(admin.model, f):
                 yield f
 
     def visible(modeladmin, request):
@@ -85,16 +84,14 @@ def _get_all_admin_fields(request):
     for model, modeladmin in admin.site._registry.items():
         model_admins[model] = modeladmin
         if visible(modeladmin, request):
-            all_admin_fields[model].update(from_fieldsets(modeladmin, model))
+            all_admin_fields[model].update(from_fieldsets(modeladmin, True))
             all_admin_fields[model].add(_OPEN_IN_ADMIN)
 
             # check the inlines, these are already filtered for access
             for inline in modeladmin.get_inline_instances(request):
-                if inline.model not in model_admins:  # pragma: no cover
+                if inline.model not in model_admins:
                     model_admins[inline.model] = inline
-                all_admin_fields[inline.model].update(
-                    from_fieldsets(inline, inline.model)
-                )
+                all_admin_fields[inline.model].update(from_fieldsets(inline, False))
                 all_admin_fields[inline.model].add(
                     _get_foreign_key(model, inline.model, inline.fk_name).name
                 )
@@ -111,28 +108,41 @@ def _get_fields_for_model(model, model_admins, admin_fields):
     fields = {}
     fks = {}
 
+    model_name = get_model_name(model)
     model_fields = {f.name: f for f in model._meta.get_fields()}
 
     for field_name in admin_fields[model]:
         field = model_fields.get(field_name)
         if field_name == _OPEN_IN_ADMIN:
-            fields[_OPEN_IN_ADMIN] = OrmField(type_=HTMLFieldType, concrete=False)
+            fields[_OPEN_IN_ADMIN] = OrmField(
+                type_=HTMLFieldType, concrete=False, model_name=model_name
+            )
         elif isinstance(field, (ForeignObjectRel, models.ManyToManyField)):
             pass  # TODO 2many support
         elif isinstance(field, models.ForeignKey):
             if field.related_model in admin_fields:
                 fks[field_name] = OrmFkField(get_model_name(field.related_model))
         elif isinstance(field, type(None)):
-            fields[field_name] = OrmField(type_=StringFieldType, concrete=False)
+            fields[field_name] = OrmField(
+                type_=StringFieldType, concrete=False, model_name=model_name
+            )
         else:
-            for django_types, field_type in _FIELD_MAP:
-                if isinstance(field, django_types):
-                    if field_type:
-                        fields[field_name] = OrmField(type_=field_type, concrete=True)
-                    break
+            if field.__class__ in _FIELD_MAP:
+                field_type = _FIELD_MAP[field.__class__]
+            else:
+                for django_type, field_type in _FIELD_MAP.items():
+                    if isinstance(field, django_type):
+                        break
+                else:
+                    field_type = None
+
+            if field_type:
+                fields[field_name] = OrmField(
+                    type_=field_type, concrete=True, model_name=model_name
+                )
             else:  # pragma: no cover
-                print(
-                    f"DataBrowser: {model.__name__}.{field_name} unknown type {type(field).__name__}"
+                logging.getLogger(__name__).warning(
+                    f"{model.__name__}.{field_name} unsupported type {type(field).__name__}"
                 )
 
     return OrmModel(fields=fields, fks=fks, admin=model_admins[model])
@@ -166,7 +176,7 @@ def _get_django_lookup(field_type, lookup):
         return lookup
 
 
-def get_data(request, bound_query):
+def get_results(request, bound_query):
     request.data_browser = True
 
     if not bound_query.fields:
@@ -202,12 +212,12 @@ def get_data(request, bound_query):
 
     # no calculated fields early out using qs.values
     if not bound_query.calculated_fields:
-        data = []
+        results = []
         for row in qs.values(*[f.path for f in bound_query.fields]).distinct():
-            data.append(
+            results.append(
                 [field.type_.format(row[field.path]) for field in bound_query.fields]
             )
-        return data
+        return results
 
     # preloading
     select_related = set()
@@ -243,22 +253,27 @@ def get_data(request, bound_query):
         url = reverse(url_name, args=[obj.pk])
         return f'<a href="{url}">{obj}</a>'
 
-    # get data
-    def lookup(obj, path):
+    # get results
+    def lookup(obj, field):
         value = obj
-        for part in path.split("__"):
-            if part == _OPEN_IN_ADMIN:
-                value = get_admin_link(value)
-            else:
-                value = getattr(value, part, None)
-        return value() if callable(value) else value
+        *parts, tail = field.path.split("__")
+        for part in parts:
+            value = getattr(value, part, None)
 
-    data = []
+        admin = bound_query.orm_models[field.orm_field.model_name].admin
+        if field.concrete:
+            return getattr(value, tail)
+        elif tail == _OPEN_IN_ADMIN:
+            return get_admin_link(value)
+        elif hasattr(admin, tail):
+            return getattr(admin, tail)(value)
+        else:
+            value = getattr(value, tail)
+            return value() if callable(value) else value
+
+    results = []
     for row in qs.distinct():
-        data.append(
-            [
-                field.type_.format(lookup(row, field.path))
-                for field in bound_query.fields
-            ]
+        results.append(
+            [field.type_.format(lookup(row, field)) for field in bound_query.fields]
         )
-    return data
+    return results
