@@ -6,7 +6,6 @@ import sys
 import django.contrib.admin.views.decorators as admin_decorators
 from django import http
 from django.conf import settings
-from django.http import HttpResponse, JsonResponse
 from django.shortcuts import get_object_or_404
 from django.template import engines, loader
 from django.template.response import TemplateResponse
@@ -14,10 +13,10 @@ from django.urls import reverse
 from django.utils import timezone
 from django.views.decorators.csrf import csrf_exempt
 
-from .models import View
-from .orm import _OPEN_IN_ADMIN, get_data, get_models
-from .query import TYPES, BoundQuery, Query
 from . import version
+from .models import View
+from .orm import _OPEN_IN_ADMIN, get_models, get_results
+from .query import TYPES, BoundQuery, Query
 
 
 def _get_query_data(bound_query):
@@ -39,7 +38,7 @@ def _get_query_data(bound_query):
     }
 
 
-def _get_config(orm_models):
+def _get_config(user, orm_models):
     types = {
         name: {
             "lookups": {n: {"type": t} for n, t in type_.lookups.items()},
@@ -55,7 +54,7 @@ def _get_config(orm_models):
         front = {"id": 1, _OPEN_IN_ADMIN: 2}
         return sorted(fields, key=lambda x: front.get(x, sys.maxsize))
 
-    orm_models = {
+    all_model_fields = {
         model_name: {
             "fields": {
                 name: {"type": orm_field.type_.name, "concrete": orm_field.concrete}
@@ -71,13 +70,29 @@ def _get_config(orm_models):
         for model_name, orm_model in orm_models.items()
     }
 
+    saved_views = [
+        {
+            "name": view.name,
+            "public": view.public,
+            "model": view.model_name,
+            "description": view.description,
+            "query": _get_query_data(BoundQuery(view.get_query(), orm_models)),
+        }
+        for view in View.objects.filter(owner=user).order_by("name")
+    ]
+
+    admin_url = None
+    if "data_browser.View" in orm_models:
+        admin_url = reverse(f"admin:{View._meta.db_table}_add")
+
     return {
         "baseUrl": reverse("data_browser:home"),
-        "adminUrl": reverse(f"admin:{View._meta.db_table}_add"),
+        "adminUrl": admin_url,
         "types": types,
-        "allModelFields": orm_models,
-        "sortedModels": sorted(orm_models),
+        "allModelFields": all_model_fields,
+        "sortedModels": sorted(all_model_fields),
         "version": version,
+        "savedViews": saved_views,
     }
 
 
@@ -85,11 +100,11 @@ def _get_context(request, model_name, fields):
     query = Query.from_request(model_name, fields, request.GET)
     orm_models = get_models(request)
     if query.model_name and query.model_name not in orm_models:
-        raise http.Http404(f"query.model_name does not exist")
+        raise http.Http404(f"{query.model_name} does not exist")
     bound_query = BoundQuery(query, orm_models)
     return {
-        "config": _get_config(orm_models),
-        "initialState": {"data": [], **_get_query_data(bound_query)},
+        "config": _get_config(request.user, orm_models),
+        "initialState": {"results": [], **_get_query_data(bound_query)},
         "sentryDsn": getattr(settings, "DATA_BROWSER_FE_DSN", None),
     }
 
@@ -97,7 +112,7 @@ def _get_context(request, model_name, fields):
 @admin_decorators.staff_member_required
 def query_ctx(request, *, model_name, fields=""):  # pragma: no cover
     ctx = _get_context(request, model_name, fields)
-    return JsonResponse(ctx)
+    return http.JsonResponse(ctx)
 
 
 @admin_decorators.staff_member_required
@@ -118,38 +133,40 @@ def query_html(request, *, model_name="", fields=""):
 @admin_decorators.staff_member_required
 def query(request, *, model_name, fields="", media):
     query = Query.from_request(model_name, fields, request.GET)
-    return _data_response(request, query, media)
+    return _data_response(request, query, media, meta=True)
 
 
 def view(request, pk, media):
     view = get_object_or_404(View.objects.filter(public=True), pk=pk)
     request.user = view.owner  # public views are run as the person who owns them
+    if not request.user.has_perm("data_browser.make_view_public"):
+        raise http.Http404("No View matches the given query.")
     query = view.get_query()
-    return _data_response(request, query, media)
+    return _data_response(request, query, media, meta=False)
 
 
-def _data_response(request, query, media):
+def _data_response(request, query, media, meta):
     orm_models = get_models(request)
     if query.model_name not in orm_models:
-        raise http.Http404(f"query.model_name does not exist")
+        raise http.Http404(f"{query.model_name} does not exist")
     bound_query = BoundQuery(query, orm_models)
-    data = get_data(request, bound_query)
+    results = get_results(request, bound_query)
 
     if media == "csv":
         buffer = io.StringIO()
         writer = csv.writer(buffer)
         writer.writerow(f.path for f in bound_query.fields)
-        writer.writerows(data)
+        writer.writerows(results)
         buffer.seek(0)
-        response = HttpResponse(buffer, content_type="text/csv")
+        response = http.HttpResponse(buffer, content_type="text/csv")
         response[
             "Content-Disposition"
         ] = f"attachment; filename={query.model_name}-{timezone.now().isoformat()}.csv"
         return response
     elif media == "json":
-        resp = _get_query_data(bound_query)
-        resp["data"] = data
-        return JsonResponse(resp)
+        resp = _get_query_data(bound_query) if meta else {}
+        resp["results"] = results
+        return http.JsonResponse(resp)
     else:
         assert False
 
